
#include <stdio.h>
#include "pthread_flag.h"


struct pthread_flag_s {
	uint32_t flag;
	pthread_mutex_t mutex;
};

static struct pthread_flag_s flags[PTHREAD_FLAG_POOL];
static int flags_used = 0;

pthread_flag_t *pthread_flag_init(uint32_t initial_value)
{
    pthread_flag_t *pflag;
    int result;
    
    if (flags_used >= PTHREAD_FLAG_POOL) {
		printf("Error[pthread_flag_init]: pool exhausted\n");
        return NULL;
    }
    
    pflag = &flags[flags_used++];
	pflag->flag = initial_value;
    result = pthread_mutex_init(&pflag->mutex, NULL);
    if (result != 0) {
		printf("Error[pthread_flag_init]: mutex_init error %d\n", result);
        return NULL;
    }
    return pflag;
}

void pthread_flag_set(pthread_flag_t *pflag)
{
	pthread_mutex_lock(&pflag->mutex);
	pflag->flag = 1;
	pthread_mutex_unlock(&pflag->mutex);
}

void pthread_flag_clear(pthread_flag_t *pflag)
{
	pthread_mutex_lock(&pflag->mutex);
	pflag->flag = 0;
	pthread_mutex_unlock(&pflag->mutex);
}

uint32_t pthread_flag_get(pthread_flag_t *pflag)
{
	uint32_t flag_value;
	pthread_mutex_lock(&pflag->mutex);
	flag_value = pflag->flag;
	pthread_mutex_unlock(&pflag->mutex);
	return flag_value;
}

void pthread_flag_destroy(pthread_flag_t *pflag)
{
	pthread_mutex_destroy(&pflag->mutex);
}
